#!/bin/sh
#############################################################################
#
# Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
#  ---All Rights Reserved.
#
#############################################################################
#
# Description:
#     Shell script to execute a SAS Fraud Management job
#
# Usage:
#     bat_run_job.sh -j <jobid> [-s <sysparm>]
#
#     where:
#
#     <jobid>     JOB_ID or JOB_ALIAS from the job definition file
#                 !SASROOT/misc/fsmcm/data/fsm_job.xml
#     <sysparm>   An optional string for passing to invoked batch job
#
# Environment:
#     The following environment variables must be set for this script to
#     run:
#        SASFRAUD_HOME
#        IMPLEMENT_FOLDER
#
# Exit Status:
#     0  Success
#    >0  Failure
#
#############################################################################
#set -x
sysparm="";
SASes="0"
submittal="";
pid=$$

# define where dynamic scripts are created
export tempdir="/tmp"

function errtrap {

   typeset es=$?
   
   echo "Submittal type is $submittal, line number=$1, es=$es" >> ${LOG_FILE} 

   if [ "$submittal" = "Job lookup" ]
   then
      SASes=$es
      echo "WARNING line $1: $submittal exited with status $SASes." >> ${LOG_FILE}
   elif [ "$submittal" = "Run job" ]
   then
      SASes=$es
      echo "WARNING line $1: $submittal exited with status $SASes." >> ${LOG_FILE}
   elif [ "$submittal" = "Submit job 2" ]
   then
      echo "WARNING line $1: $submittal exited with status $es." >> ${LOG_FILE}
   else
      echo "trap not from SAS exec es=$es."  >> ${LOG_FILE} 
      echo "WARNING line $1: trapped error $es." >> ${LOG_FILE} 
   fi
   }

trap 'errtrap $LINENO' ERR
#
# Source the user's profile ( cron doesn't necessarily do this )
# The assumption is that the above puts sas in path and defines the needed
# environment variables
#
#. ${HOME}/.profile

#
# Prints the usage of this script
#
print_usage() {
   echo ""
   echo "Usage: $0  -j <job id> -s <sysparm>"
   echo "where:"
   echo "\t<jobid> is the JOB_ID or JOB_ALIAS of the job to execute "
   echo "\t<sysparm> an optional string to be passed to invoked batch job "
   echo ""
}


#
# Parse the command line arguments
# -d no longer used but remains for informational purposes
#

logfolder="common";

while getopts j:d:s: opt; do
   case $opt in
      j)    jobid=$OPTARG;;
      d)    db=$OPTARG;;
      s)    sysparm=$OPTARG;;
     \?)    print_usage
            exit ;;
   esac
done


sas_pgm="bat_run_job2.sas";

export app_implroot="$SASFRAUD_HOME/$IMPLEMENT_FOLDER";

LOG_TIME=`date +%H%M%S`

######################################################
# Generate the file name for this shell script's log:
######################################################
LOG_DIR=$app_implroot/common/logs
LOG_DATE=`date +"%y_%m_%d"`
LOG_FTIME=`date +%H_%M`
LOG_FNAME=${LOG_DATE}_${LOG_FTIME}_job_${jobid}_shell_${pid}.log
LOG_FILE=${LOG_DIR}/${LOG_FNAME}
export LOG_FILE


#
# Log script start
#

echo "Job {$jobid} submitted: ${LOG_DATE} ${LOG_TIME}" >> ${LOG_FILE}

#
# Ensure that our environment knows about SAS
#
if [ ! -x `which sas_u8` ]; then
   echo "$0: Error: sas_u8 not found or not executable"  >> ${LOG_FILE}
   exit 3
fi


#
# Verify that the job ID or alias is not null
#

if [ -z "${jobid}" ]; then
   echo "The -j parameter was not specified!" >> ${LOG_FILE}
   exit 3
fi


#
# Invoke the SAS to lookup the job definition of requested job ID or alias
#
submittal="Job lookup";
scriptparm=$(sas_u8 -sysparm "pid=$pid job=$jobid impl=$IMPLEMENT_FOLDER BATCHINIT sysparm=$sysparm" -echoauto -sysin "$SAS_ROOT/misc/fsmcm/source/$sas_pgm" -LOGAPPLNAME "common" -LOG "job_${jobid}_lookup_${pid}.log" -logconfigloc "$SAS_ROOT/misc/fsmcm/source/logconfig.trace.xml" -noterminal -unbuflog -nosyntaxcheck) 

wait
submittal="";

echo "returned scriptparm=$scriptparm" >> ${LOG_FILE}

if [ -x ${tempdir}/submit${jobid}_${pid}.sh ]
then

   #  requested job has dynamically created script to execute for running actual job
   submittal="Run job";
   scan_parms=$(${tempdir}/submit${jobid}_${pid}.sh)
   wait
   submittal="";

   #scan_parms has info for running the job log scanner
   echo "scan_parms=$scan_parms" >> ${LOG_FILE}

   scan_parms2=`expr "$scan_parms" : ".*\(-q $jobid .*\) ~~.*"`
   echo "scan_parms2=$scan_parms2" >> ${LOG_FILE}
   scan_parms="$scan_parms2"

   #delete the first dynamically created script
   rm -f ${tempdir}/submit${jobid}_${pid}.sh

   echo "Executing job 2." >> ${LOG_FILE}

   #start job scanner process passing info about actual job and its log
   submittal="Submit job 2";
   bat_scan_log2.sh $scan_parms

else
   echo "Did not execute submit${jobid}_${pid}.sh." >> ${LOG_FILE}
   exit 3
fi


#
# Log script complete
#
echo "Exiting with SAS rc=$SASes" >> ${LOG_FILE}
echo "Submit End: `date +"%y_%m_%d"` `date +%H%M%S`" >> ${LOG_FILE}
echo "" >> ${LOG_FILE}

exit $SASes

#
# End of File
#
###################################################################
