
CREATE OR REPLACE PROCEDURE RECLASSIFY_ALERT (
   I_SMH_ACCT_TYPE           IN     VARCHAR2,
   I_SMH_ACTIVITY_TYPE       IN     VARCHAR2,
   I_CMX_TRAN_ID             IN     NUMBER,
   I_ALERT_TYPE              IN     VARCHAR2,
   I_ALERT_VALUE             IN     VARCHAR2,
   I_PARENT_ALERT_TYPE       IN     VARCHAR2, --- If NO VALUE, there is no parent to set
   I_PARENT_VALUE            IN     VARCHAR2,
   I_CONTACT_TYPE            IN     VARCHAR2, --- Will always have a value, but may not follow hierchary. The UI will validate the field is on the txn
   I_CONTACT_VALUE           IN     VARCHAR2,
   I_MULTI_ORG_ID            IN     INTEGER,
   I_CMX_CLIENT_AMT          IN     BINARY_DOUBLE,
   I_CMX_MOD_AMT             IN     BINARY_DOUBLE,
   I_RQO_TRAN_DATE           IN     DATE,
   I_RQO_TRAN_TIME           IN     TIMESTAMP,
   I_RQO_PROC_UTC_DATETIME   IN     TIMESTAMP,
   I_RRR_SCORE_1             IN     INTEGER,
   I_RRR_SCORE_2             IN     INTEGER,
   I_RRR_SCORE_3             IN     INTEGER,
   I_RRR_SCORE_4             IN     INTEGER,
   I_QUEUE_ID                IN     NUMBER,
   I_SRC_ALERT_ID            IN     NUMBER,
   I_SRC_ALERT_TYPE          IN     VARCHAR2,
   I_SRC_ALERT_VERSION_NBR   IN     INTEGER,
   I_USER_NAME               IN     VARCHAR2,
   I_NEED_TO_COMMIT          IN     VARCHAR2,
   o_getalertcur                OUT SYS_REFCURSOR)
-----------------------------------------------------------------------------
-- file:  RECLASSIFY_ALERT
--
--
-- description:
--    Creates a new Alert based on an existing alert. The UI is validating the USER_NAME has access to the ALERT_TYPE
--    being created. The proc needs to validate access to the MO of the txn.
--1.    Look up the field name for the alert type.
--2.    Look up the field name for the parent alert type.
--3.    Look up the field name for the contact type
--6.    Look up  GMT Offset based on contact value.
--7.    If alert does not exist, insert alert
--8.    If alert exists and not closed, do nothing
--9.    If alert exists and is closed, reinit
--10.    Insert alert action
--11.    Return alert ID on success
--12.   Update txn with marking
--13.   Insert fmx_fraud_updt
--14.    Throw exception on failure.
--
-- calls:
--     get_alert
--     set_xact_fraud_flag
--     save_alert_Action
--
-- notes:
--
--
-- usage:
--        Called from the web UI
--
--
-- history:
--     2012/07/23    leushe                 Ported from DB2
--     2014/03/17    leushe      Added reference for FRAUD_SCORE_2-4 and remove FRAUD_REASON_CODES
--     2015/01/27    LEUSHE     set FINAL_DISPOSITION_TIMESTAMP=null when performing the update on the alert
--     2016/01/25    leushe      changed signature of call to save_alert_Action to include alert_status and action_type
--     2016/06/20    leushe      Added support for new columns LST_SERVICE_USER and EXPIRATION_TIMESTAMP 
--     2016/07/05    leushe      Added support for new columns LST_DISPOSITION_USER and LST_DISPOSITION_TIMESTAMP, remove support for new columns LST_SERVICE_USER
--     2017/02/07     leushe      On reinit, set LSTUPDT_NBR to 0 
------------------------------------------------------------------------
IS
   SQLSTATE                  VARCHAR2 (5) DEFAULT '00000';
   L_SQLSTATE                VARCHAR2 (5) DEFAULT '00000';
   L_ROW_COUNT               INTEGER DEFAULT -1;
   L_CURRENT_UTC_TIMESTAMP   TIMESTAMP;
   L_ALERT_ACTION_STATUS     VARCHAR2 (10) DEFAULT '';
   L_TENANT_MULTI_ORG_ID     INTEGER;
   L_ALERT_ID                NUMBER (19) DEFAULT -1;
   L_ALERT_VERSION_NBR       INTEGER DEFAULT 0;
   L_ALERT_STATUS            VARCHAR2 (10) DEFAULT 'ERROR';
   L_BASESQL                 VARCHAR2 (2500);
   err_code                  INTEGER;
   err_msg                   VARCHAR2 (2000);


   ALERTUPDATEFAILED         EXCEPTION;
   ALERTCREATEFAILED         EXCEPTION;



   TYPE refcur IS REF CURSOR;

   c_stmt                    refcur;
BEGIN
   --Set the current timestamp once
   L_CURRENT_UTC_TIMESTAMP := SYS_EXTRACT_UTC (CURRENT_TIMESTAMP);

   SELECT tenant_multi_org_id
     INTO l_TENANT_multi_org_id
     FROM fcm_multi_org
    WHERE multi_org_id = i_multi_org_id;

   /*if we didn't set l_cmx_tran_id in the validation query above, bail*/
   l_baseSql :=
      'select alert_id ,alert_version_nbr,alert_status,tenant_multi_org_id from FCM_ALERT where tenant_multi_org_id = :1 and alert_type = :2 and alert_value = :3';



   OPEN c_stmt FOR l_baseSql
      USING l_TENANT_multi_org_id, i_alert_type, I_ALERT_VALUE;

   FETCH c_stmt
      INTO l_alert_id,
           l_alert_version_nbr,
           l_alert_status,
           l_TENANT_multi_org_id;

   IF (c_stmt%FOUND)
   THEN
      l_sqlstate := '00000';
   ELSE
      l_sqlstate := '99999';
   END IF;

   CLOSE c_stmt;



   IF (l_sqlstate = '00000')
   THEN
      IF (l_alert_status = 'CLOSED')
      THEN
         l_alert_version_nbr := l_alert_version_nbr + 1;
         l_alert_action_status := 'INITCLOSED';

         UPDATE FCM_ALERT
            SET LSTUPDT_NBR = 0,
                TENANT_MULTI_ORG_ID = l_TENANT_multi_org_id,
                SRC_ALERT_ID = I_SRC_ALERT_ID,
                SRC_ALERT_VERSION_NBR = I_SRC_ALERT_VERSION_NBR,
                SRC_ALERT_TYPE = I_SRC_ALERT_TYPE,
                INIT_CMX_TRAN_ID = I_CMX_TRAN_ID,
                INIT_SMH_ACCT_TYPE = I_SMH_ACCT_TYPE,
                INIT_SMH_ACTIVITY_TYPE = I_SMH_ACTIVITY_TYPE,
                CONTACT_VALUE = I_CONTACT_VALUE,
                PARENT_ALERT_VALUE = I_PARENT_VALUE,
                CONTACT_TYPE = I_CONTACT_TYPE,
                PARENT_ALERT_TYPE = I_PARENT_ALERT_TYPE,
                RESULT_ID = NULL,
                ALERT_STATUS = 'ACTIVE',
                ALERT_ORIGIN = 'INBOUND',
                BASE_RULE_ID = NULL,
                RULE_VERSION_NBR = NULL,
                ALERT_SERVICE_CNT = 0,
                MULTI_ORG_ID = I_MULTI_ORG_ID,
                FINAL_DISPOSITION_TIMESTAMP = NULL,
				EXPIRATION_TIMESTAMP          = to_timestamp ('9999-12-31 00.00.00','yyyy-mm-dd hh24:mi:ss'),
                LST_TRANSACTION_TIMESTAMP =
                   (CASE
                       WHEN (   i_rqo_tran_date IS NULL
                             OR i_rqo_tran_Time IS NULL)
                       THEN
                          i_rqo_proc_utc_datetime
                       ELSE
                          TO_TIMESTAMP (
                                TO_CHAR (i_rqo_tran_date)
                             || ' '
                             || (TO_CHAR (i_rqo_tran_time, 'hh:mi:ss')))
                    END),
                INIT_TRANSACTION_TIMESTAMP =
                   (CASE
                       WHEN (   i_rqo_tran_date IS NULL
                             OR i_rqo_tran_Time IS NULL)
                       THEN
                          i_rqo_proc_utc_datetime
                       ELSE
                          TO_TIMESTAMP (
                                TO_CHAR (i_rqo_tran_date)
                             || ' '
                             || (TO_CHAR (i_rqo_tran_time, 'hh:mi:ss')))
                    END),
                LST_TRANSACTION_AMT = I_CMX_CLIENT_AMT,
                INIT_TRANSACTION_AMT = I_CMX_CLIENT_AMT,
                LST_TRANSACTION_MOD_AMT = I_CMX_MOD_AMT,
                INIT_TRANSACTION_MOD_AMT = I_CMX_MOD_AMT,
                FRAUD_SCORE_1 = I_RRR_SCORE_1,
                HWM_FRAUD_SCORE_1 = I_RRR_SCORE_1,
                INIT_FRAUD_SCORE_1 = I_RRR_SCORE_1,
                FRAUD_SCORE_2 = I_RRR_SCORE_2,
                HWM_FRAUD_SCORE_2 = I_RRR_SCORE_2,
                INIT_FRAUD_SCORE_2 = I_RRR_SCORE_2,
                FRAUD_SCORE_3 = I_RRR_SCORE_3,
                HWM_FRAUD_SCORE_3 = I_RRR_SCORE_3,
                INIT_FRAUD_SCORE_3 = I_RRR_SCORE_3,
                FRAUD_SCORE_4 = I_RRR_SCORE_4,
                HWM_FRAUD_SCORE_4 = I_RRR_SCORE_4,
                INIT_FRAUD_SCORE_4 = I_RRR_SCORE_4,
                PAUSE_TIMESTAMP = NULL,
                PAUSE_ELAPSED_TIME = NULL,
                QUEUE_ID = I_QUEUE_ID,
                ALERT_VERSION_NBR = l_alert_version_nbr,
                CHECKOUT_TIMESTAMP = NULL,
				LST_DISPOSITION_USER 	= null,
				LST_DISPOSITION_TIMESTAMP=null,
                CREATE_USER = I_USER_NAME,
                CREATE_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP,
                LSTUPDT_USER = I_USER_NAME,
                LSTUPDT_TIMESTAMP = L_CURRENT_UTC_TIMESTAMP
          WHERE ALERT_ID = l_ALERT_ID;

         IF (SQL%ROWCOUNT = 1)
         THEN
            l_sqlstate := '00000';
         ELSE
            RAISE alertUpdateFailed;
         END IF;
      END IF;
   ELSE
      l_alert_version_nbr := 0;

      SELECT alert_seq.NEXTVAL INTO l_alert_id FROM DUAL;

      l_alert_action_status := 'ALERTINIT';

      INSERT INTO fcm_alert (TENANT_MULTI_ORG_ID,
                             ALERT_VERSION_NBR,
                             ALERT_VALUE,
                             ALERT_TYPE,
                             MULTI_ORG_ID,
                             CONTACT_VALUE,
                             PARENT_ALERT_VALUE,
                             QUEUE_ID,
                             ALERT_STATUS,
                             GMT_OFFSET,
                             FRAUD_SCORE_1,
                             HWM_FRAUD_SCORE_1,
                             INIT_FRAUD_SCORE_1,
                             FRAUD_SCORE_2,
                             HWM_FRAUD_SCORE_2,
                             INIT_FRAUD_SCORE_2,
                             FRAUD_SCORE_3,
                             HWM_FRAUD_SCORE_3,
                             INIT_FRAUD_SCORE_3,
                             FRAUD_SCORE_4,
                             HWM_FRAUD_SCORE_4,
                             INIT_FRAUD_SCORE_4,
                             ALERT_SERVICE_CNT,
                             ALERT_ORIGIN,
                             INIT_TRANSACTION_TIMESTAMP,
                             INIT_TRANSACTION_AMT,
                             LST_TRANSACTION_AMT,
                             LST_TRANSACTION_TIMESTAMP,
                             INIT_TRANSACTION_MOD_AMT,
                             LST_TRANSACTION_MOD_AMT,
                             NEXT_SERVICE_TIMESTAMP,
                             QUEUE_ENTRY_TIMESTAMP,
                             CREATE_USER,
                             CREATE_TIMESTAMP,
                             LSTUPDT_USER,
                             LSTUPDT_TIMESTAMP,
                             LSTUPDT_NBR,
                             LOGICAL_DEL_IND,
                             INIT_CMX_TRAN_ID,
                             INIT_SMH_ACCT_TYPE,
                             INIT_SMH_ACTIVITY_TYPE,
                             CONTACT_TYPE,
                             PARENT_ALERT_TYPE,
                             SRC_ALERT_ID,
                             SRC_ALERT_VERSION_NBR,
                             SRC_ALERT_TYPE,
                             ALERT_ID)
              VALUES (
                        l_TENANT_multi_org_id,
                        l_alert_version_nbr,
                        I_ALERT_VALUE,
                        i_alert_type,
                        i_multi_org_id,
                        i_contact_value,
                        I_PARENT_VALUE,
                        i_queue_id,
                        'ACTIVE',
                        0,
                        i_rrr_score_1,
                        i_rrr_score_1,
                        i_rrr_score_1,
                        i_rrr_score_2,
                        i_rrr_score_2,
                        i_rrr_score_2,
                        i_rrr_score_3,
                        i_rrr_score_3,
                        i_rrr_score_3,
                        i_rrr_score_4,
                        i_rrr_score_4,
                        i_rrr_score_4,
                        0,
                        'INBOUND',
                        (CASE
                            WHEN (   i_rqo_tran_date IS NULL
                                  OR i_rqo_tran_Time IS NULL)
                            THEN
                               i_rqo_proc_utc_datetime
                            ELSE
                               TO_TIMESTAMP (
                                     TO_CHAR (i_rqo_tran_date)
                                  || ' '
                                  || (TO_CHAR (i_rqo_tran_time, 'hh:mi:ss')))
                         END),
                        i_cmx_client_amt,
                        i_cmx_client_amt,
                        (CASE
                            WHEN (   i_rqo_tran_date IS NULL
                                  OR i_rqo_tran_Time IS NULL)
                            THEN
                               i_rqo_proc_utc_datetime
                            ELSE
                               TO_TIMESTAMP (
                                     TO_CHAR (i_rqo_tran_date)
                                  || ' '
                                  || (TO_CHAR (i_rqo_tran_time, 'hh:mi:ss')))
                         END),
                        i_cmx_mod_amt,
                        i_cmx_mod_amt,
                        L_CURRENT_UTC_TIMESTAMP,
                        L_CURRENT_UTC_TIMESTAMP,                        
                        I_USER_NAME,
                        L_CURRENT_UTC_TIMESTAMP,
                        I_USER_NAME,
                        L_CURRENT_UTC_TIMESTAMP,
                        0,
                        '0',
                        i_cmx_tran_id,
                        I_SMH_ACCT_TYPE,
                        I_SMH_ACTIVITY_TYPE,
                        I_contact_type,
                        i_parent_alert_type,
                        I_SRC_ALERT_ID,
                        I_SRC_ALERT_VERSION_NBR,
                        I_SRC_ALERT_TYPE,
                        l_alert_id);

      IF (SQL%ROWCOUNT = 1)
      THEN
         l_sqlstate := '00000';
      ELSE
         RAISE alertCreateFailed;
      END IF;
   END IF;



   IF (LENGTH (L_ALERT_ACTION_STATUS) > 0)
   THEN
      save_alert_Action (l_alert_id,
                         L_ALERT_ACTION_STATUS,
                         'OUT',
                         'N');
   END IF;


   IF (I_NEED_TO_COMMIT = 'Y')
   THEN
      COMMIT;
   END IF;

   GET_ALERT (l_alert_id, i_user_name, o_getalertcur);
-- eventually will need to signal an exception; now just return an empty set
--  ELSE
--     GET_ALERT(-1, i_user_name,o_getalertcur);
--  END IF;
EXCEPTION
   WHEN alertCreateFailed
   THEN
      RAISE_APPLICATION_ERROR (-1, 'Failed to Create new Alert');
   WHEN alertUpdateFailed
   THEN
      RAISE_APPLICATION_ERROR (-1, 'Failed to Re-Init Alert');
   WHEN OTHERS
   THEN
      BEGIN
         err_code := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);
      --- INSERT INTO audit_table (error_number, error_message) values (err_code,err_msg);

      END;
END RECLASSIFY_ALERT;
/
